/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.engine.interpreter;

import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import tools.descartes.dql.connector.ModelStructureQueryConnector;
import tools.descartes.dql.connector.QueryProgressListener;
import tools.descartes.dql.lang.descartesQL.EntityReference;
import tools.descartes.dql.lang.descartesQL.EntityReferenceClause;
import tools.descartes.dql.lang.descartesQL.ListDoFQuery;
import tools.descartes.dql.lang.descartesQL.ListEntitiesQuery;
import tools.descartes.dql.lang.descartesQL.ListMetricsQuery;
import tools.descartes.dql.lang.descartesQL.ListQuery;
import tools.descartes.dql.lang.descartesQL.ModelFamily;
import tools.descartes.dql.lang.descartesQL.ModelReferenceClause;
import tools.descartes.dql.lang.descartesQL.ModelStructureQuery;
import tools.descartes.dql.lang.descartesQL.UsingClause;
import tools.descartes.dql.lang.descartesQL.WithClause;
import tools.descartes.dql.models.mapping.domain.DomainFactory;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;
import tools.descartes.dql.models.mapping.mapping.MappingFactory;
import tools.descartes.dql.models.mapping.mapping.impl.MappingFactoryImpl;

/**
 * Interpreter for LIST queries
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public class ModelStructureQueryInterpreter
		extends AbstractQueryInterpreter<ModelStructureQuery, ModelStructureQueryConnector> {

	private final MappingFactory mappingFactory = MappingFactory.eINSTANCE;
	private final DomainFactory domainFactory = DomainFactory.eINSTANCE;

	public ModelStructureQueryInterpreter(final ModelStructureQuery query) {
		super(query);
	}

	public ModelStructureQueryInterpreter(final ModelStructureQuery query, List<QueryProgressListener> listeners) {
		super(query, listeners);
	}

	@Override
	protected Class<ModelStructureQueryConnector> getQueryConnectorClass() {
		return ModelStructureQueryConnector.class;
	}

	/**
	 * Interpret query as LIST DOF
	 * 
	 * @param query
	 *            The ListQuery to be down casted
	 * @return True, if interpreting was successful
	 */
	private boolean interpretListDoFQuery(final ListQuery query) {
		final UsingClause usingClause = query.getUsingClause();
		final ListDoFQuery doFQuery = (ListDoFQuery) query.getQueryType();
		final ModelReferenceClause modelRefClause = usingClause.getModelReference();

		if (!getQueryConnector().supportsDoFRequests()) {
			log.error("Connector does not support DoF requests, aborting.");
			return false;
		}

		final MappingFactory mappingFactory = MappingFactoryImpl.init();
		final EntityMapping request = mappingFactory.createEntityMapping();

		request.setQueryAsString(getQueryAsString());
		request.setModelLocation(modelRefClause.getModel().getLocation());

		final WithClause withClause = doFQuery.getWithClause();

		if (withClause != null) {
			request.setDoFModelLocation(withClause.getModelReference().toString());
		}

		setResult(getQueryConnector().requestDoF(request));

		return true;
	}

	/**
	 * Interpret query as LIST ENTITIES
	 * 
	 * @param query
	 *            The ListQuery to be down casted
	 * @return True, if interpreting was successful
	 */
	private boolean interpretListEntitiesQuery(final ListQuery query) {
		// TODO: Implement
		// FilterClause filterClause = ((ListQuery) query).getFilterClause();
		final UsingClause usingClause = query.getUsingClause();
		final ModelReferenceClause modelRefClause = usingClause.getModelReference();

		if (!getQueryConnector().supportsEntityRequests()) {
			log.error("Connector does not support entity requests, aborting.");
			return false;
		}

		final EntityMapping request = mappingFactory.createEntityMapping();
		request.setModelLocation(modelRefClause.getModel().getLocation());

		// TODO: Interpret FilterClause

		// Delegate query to connector
		setResult(getQueryConnector().requestEntities(request));

		return true;
	}

	/**
	 * Interpret query as LIST METRICS
	 * 
	 * @param query
	 *            The ListQuery to be down casted
	 * @return True, if interpreting was successful
	 */
	private boolean interpretListMetricsQuery(final ListQuery query) {
		// TODO: Implement
		// FilterClause filterClause = ((ListQuery) query).getFilterClause();
		DefaultProvider defaultProvider = new DefaultProvider(query.getUsingClause());
		final ListQuery listQuery = query;
		final ListMetricsQuery listMetricsQuery = (ListMetricsQuery) query.getQueryType();

		final UsingClause usingClause = listQuery.getUsingClause();
		final ModelReferenceClause modelRefClause = usingClause.getModelReference();

		final EntityMapping mapping = mappingFactory.createEntityMapping();

		if (!getQueryConnector().supportsProbeRequests()) {
			log.error("Connector does not support probe requests, aborting.");

			return false;
		}

		mapping.setModelLocation(modelRefClause.getModel().getLocation());

		final EntityReferenceClause entityReferenceClause = listMetricsQuery.getEntityReferenceClause();

		if (entityReferenceClause == null) {
			log.error("Missing entitiy reference clause. Aborting.");
			return false;
		}

		final EList<EntityReference> refClause = entityReferenceClause.getEntityReferences();
		for (final EntityReference ref : refClause) {
			Entity entity = domainFactory.createEntity();
			if (ref.getType() != null)
				entity.setEntityType(ref.getType());
			else
				entity.setEntityType(defaultProvider.lookUpEntityType(ref.getIdentifier()));
			entity.setIdentifier(ref.getIdentifier());
			if (ref.getAlias() != null) {
				entity.setAlias(ref.getAlias().getAlias());
			}
			mapping.getEntities().add(entity);
		}

		setResult(getQueryConnector().requestProbes(mapping));

		return true;
	}

	/**
	 * Entry point for interpreting a LIST query
	 * 
	 * @param query
	 *            The ListQuery to be down casted
	 * @return True, if interpreting was successful
	 */
	private boolean interpretListQuery(final ListQuery query) {
		final EObject queryType = query.getQueryType();

		// Decide which kind of LIST query is incoming and delegate its
		// processing to a suitable method
		if (queryType instanceof ListEntitiesQuery) {
			return interpretListEntitiesQuery(query);
		} else if (queryType instanceof ListMetricsQuery) {
			return interpretListMetricsQuery(query);
		} else if (queryType instanceof ListDoFQuery) {
			return interpretListDoFQuery(query);
		}

		log.error("No suitable interpreter found in interpretListQuery() for " + queryType);
		return false;
	}

	@Override
	public boolean interpretQuery() {
		final ModelStructureQuery query = getQuery();

		if (query instanceof ListQuery) {
			final ListQuery listQuery = (ListQuery) query;
			final UsingClause usingClause = listQuery.getUsingClause();
			final ModelReferenceClause modelReference = usingClause.getModelReference();
			final ModelFamily family = modelReference.getFamily();
			final String familyIdentifier = family.getFamily();

			setModelFamily(familyIdentifier);

			if (!initQueryConnector()) {
				String exception = "Could not load Connector for family " + getModelFamily() + ", aborting.";
				log.error(exception);
				throw new IllegalStateException(exception);
			}

			return interpretListQuery(listQuery);
		}

		log.error("Query type not supported");
		return false;
	}

}
