/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.platform.eclipse.launch;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import tools.descartes.dql.connector.EclipseProgressListener;
import tools.descartes.dql.connector.QueryProgressListener;
import tools.descartes.dql.core.engine.query.QueryExecutionEngine;
import tools.descartes.dql.core.engine.query.QueryExecutionEngineImpl;
import tools.descartes.dql.core.platform.eclipse.Activator;
import tools.descartes.dql.core.platform.eclipse.views.QueryResultTableViewer;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

import com.google.inject.Injector;

class QueryExecutionEngineThread implements Runnable {
	private final QueryExecutionEngine engine = new QueryExecutionEngineImpl();
	private List<EntityMapping> results;
	private IResource resource;
	private IProgressMonitor monitor;
	private CancelableEngineJob job;

	public QueryExecutionEngineThread(final IResource resource, IProgressMonitor monitor,
			CancelableEngineJob job) {
		this.monitor = monitor;
		this.resource = resource;
		this.job = job;
	}

	@Override
	public void run() {
		try {
		waitForWindowInitialization();
		
		List<QueryProgressListener> listeners = setupListeners();
		final Resource emfResource = resourceToModel(resource);
		results = engine.execute(emfResource, listeners);

		notifyAboutFinish(listeners);
		displayResults();
		} catch (RuntimeException e) {
			job.throwException(e);
		}
	}
	
	private void displayResults() {
		if (!monitor.isCanceled()) {
			// Execute in display thread (SWT-specific)
			Display.getDefault().asyncExec(new Runnable() {
				@Override
				public void run() {
					showQueryResult();
				}
			});
		}
	}

	private void notifyAboutFinish(List<QueryProgressListener> listeners) {
		for (QueryProgressListener listener : listeners) {
			listener.updateProgress(100.0f);
			listener.updateSubTask("Finished!");
		}
		monitor.done();
	}

	private List<QueryProgressListener> setupListeners() {
		EclipseProgressListener listener = new EclipseProgressListener(monitor);
		listener.updateProgress(0.0f);
		listener.updateSubTask("Startup...");
		List<QueryProgressListener> listeners = new ArrayList<QueryProgressListener>();
		listeners.add(listener);
		return listeners;
	}

	private void waitForWindowInitialization() {
		try {
			// thread needs to sleep to make sure the window is initialized
			Thread.sleep(100);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private void showQueryResult() {
		if (engine.isValidQuery()) {
			final IWorkbench workbench = PlatformUI.getWorkbench();
			final IWorkbenchWindow activeWorkbenchWindow = workbench.getActiveWorkbenchWindow();
			final IWorkbenchPage activePage = activeWorkbenchWindow.getActivePage();
			final String sourceFile = resource.getProjectRelativePath().lastSegment();

			try {
				openQueryResultTableViewer(activePage, sourceFile, results);
			} catch (final PartInitException e) {
				e.printStackTrace();
			}
		}
	}

	private void openQueryResultTableViewer(final IWorkbenchPage activePage, final String sourceFile,
			final List<EntityMapping> results) throws PartInitException {
		int resultCounter = 0;

		for (final EntityMapping result : results) {
			final String queryTitle = sourceFile + "_" + ++resultCounter;
			final IViewPart view = activePage.showView(QueryResultTableViewer.ID, queryTitle,
					IWorkbenchPage.VIEW_CREATE);
			final QueryResultTableViewer qrtv = (QueryResultTableViewer) view;
			qrtv.setQueryTitle(queryTitle);
			qrtv.setQueryResult(result);
			activePage.activate(view);
		}
	}
	
	public static Resource resourceToModel(final IResource workspaceResource) {
		// To access Xtext run-time already deployed
		final Injector injector = Activator.getDqlInjector();

		// EMF magic further obscured by dependency injection of Xtext
		final ResourceSet resourceSet = injector.getInstance(ResourceSet.class);
		final String plainAbsoluteUri = workspaceResource.getLocationURI().toString();
		final URI pathUri = URI.createURI(plainAbsoluteUri);
		final Resource resource = resourceSet.getResource(pathUri, true);

		// Finally a model instance
		return resource;
	}
}
