/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.LineBorder;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;

/**
 * Code visualising each view for each VisualizationController. It managing the
 * view of the diagram, including the control buttons below it.
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */
public class View {

	LinkedList<JComponent> addedComponents = new LinkedList<JComponent>();
	private JPanel frame = new JPanel(new BorderLayout());
	private JPanel visualizationPanel = new JPanel();
	private JPanel buttonPanel;
	private JButton changeButton;
	private JButton exportButton;;
	private ChartPanel chartPanel = new ChartPanel(null);
	private JRadioButton confidenceRadio;
	private JComboBox<String> DiagramSwitchTypes;


	public View() {
		BoxLayout layoutForViz = new BoxLayout(visualizationPanel, BoxLayout.Y_AXIS);
		visualizationPanel.setLayout(layoutForViz);
		frame.add(visualizationPanel, BorderLayout.NORTH);
		visualizationPanel.add(chartPanel);
		frame.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		frame.setVisible(true);

		buttonPanel = initializeButtons();
		frame.add(buttonPanel, BorderLayout.CENTER);
	}

	/**
	 * Method to remove all added components from the button panel
	 */
	public void reset() {
		for (JComponent comp : addedComponents)
			buttonPanel.remove(comp);

		buttonPanel.revalidate();
		buttonPanel.repaint();
		frame.revalidate();
		frame.repaint();
	}

	/**
	 * Method to initialize all nessecary Butoons (Switch, confidence, export)
	 * and adding them to a panel
	 * 
	 * @return Panel containig all three main buttons (without actionlisteners)
	 */
	private JPanel initializeButtons() {
		changeButton = new JButton("Switch Diagram");
		exportButton = new JButton("Export Diagram");
		DiagramSwitchTypes = new JComboBox<String>();
		JPanel buttons = new JPanel();
		buttons.add(changeButton);
		buttons.add(DiagramSwitchTypes);
		buttons.add(exportButton);
		return buttons;
	}

	/**
	 * @return the main frame and validate it before
	 */
	public JPanel getCompletePanel() {
		frame.validate();
		return frame;
	}

	/**
	 * Method to add String results to the GUI
	 * 
	 * @param s
	 *            String to be shown in the view
	 */
	public void setStringResult(String s) {
		JLabel labelZahl = new JLabel(s);
		labelZahl.setBackground(Color.WHITE);
		labelZahl.setOpaque(true);
		visualizationPanel.add(labelZahl);
		visualizationPanel.add(new JLabel("    "));
	}

	/**
	 * Method to add an error message to the view, if no result can be
	 * visualized by PAVO
	 */
	public void setError() {
		JLabel error = new JLabel();

		error.setText("PAVO is not able to visualize this result, sorry!");
		error.setBorder(new LineBorder(Color.RED, 5, true));
		visualizationPanel.add(error, BorderLayout.CENTER);
	}

	/**
	 * Method to uopdate chart view, centering it and set it visible
	 * 
	 * @param chart
	 *            the new chart to be shown
	 */
	public void updateChart(JFreeChart chart) {

		visualizationPanel.remove(chartPanel);
		chartPanel = new ChartPanel(chart);

		// chartPanel.setPopupMenu(popupRightClick);

		visualizationPanel.add(chartPanel, BorderLayout.CENTER);
		visualizationPanel.repaint();
		visualizationPanel.validate();
		visualizationPanel.setVisible(true);

	}

	/**
	 * method to add new diagram types to the combo box
	 * 
	 * @param s
	 *            the String to be added to the diagram switch combobox
	 */
	public void addItemToCombo(String s) {
		DiagramSwitchTypes.addItem(s);
	}

	/**
	 * Method to add new Componets to the button panel (needed for special
	 * buttons according to the diagram type)
	 * 
	 * @param comp
	 *            Component to add
	 */
	public void addComponetsToButtonPanel(JComponent comp) {
		this.addedComponents.add(comp);
		this.buttonPanel.add(comp);
	}

	public void addConfidence(JRadioButton conf) {
		this.confidenceRadio = conf;
	}

	/**
	 * Static Method to create a Panel containing RadioButtons for Minimum,
	 * maximum and mean markers including their action listener
	 * 
	 * @param minA
	 *            actionlistener for the min radio button
	 * @param maxA
	 *            actionlistener for the max radio button
	 * @param meanA
	 *            actionlistener for the mean radio button
	 * @return a panel is return containing all three Radiobuttons including
	 *         their listener
	 */
	public static JPanel createMinMaxMean(ActionListener minA, ActionListener maxA, ActionListener meanA) {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		JRadioButton minRadio = new JRadioButton("Show Minimum");
		JRadioButton maxRadio = new JRadioButton("Show Maximum");
		JRadioButton meanRadio = new JRadioButton("Show Mean");
		minRadio.addActionListener(minA);
		maxRadio.addActionListener(maxA);
		meanRadio.addActionListener(meanA);
		panel.add(minRadio);
		panel.add(maxRadio);
		panel.add(meanRadio);
		return panel;
	}

	/**
	 * Static method to create a confidence radio button, if needed
	 * 
	 * @param a
	 *            the actionlistener for this radio button
	 * @return the radiobutton including the actionlistener
	 */
	public static JRadioButton createConfidence(ActionListener a) {

		JRadioButton confidenceRadio = new JRadioButton("confidence intervals");
		confidenceRadio.addActionListener(a);

		return confidenceRadio;
	}

	/**
	 * Static method to create the comboboxes for e.g. HeatMap or BubbleChart to
	 * choose, which series shall be shown in which dimension
	 * 
	 * @param xA
	 *            ActionListener for the x combobox
	 * @param yA
	 *            ActionListener for the y combobox
	 * @param zA
	 *            ActionListener for the z combobox
	 * @return a Panel is returned, containing all three ComboBoxes inclding
	 *         their ActionListener
	 */
	public static JPanel createHeatMapControls(ActionListener xA, ActionListener yA, ActionListener zA) {
		JPanel analysisPanel = new JPanel();
		analysisPanel.setLayout(new BoxLayout(analysisPanel, BoxLayout.Y_AXIS));
		JPanel heatXPanel = new JPanel();
		heatXPanel.setLayout(new FlowLayout());
		JPanel heatYPanel = new JPanel();
		heatYPanel.setLayout(new FlowLayout());
		JPanel heatZPanel = new JPanel();
		heatZPanel.setLayout(new FlowLayout());
		heatXPanel.add(new JLabel("X:"));
		heatYPanel.add(new JLabel("Y:"));
		heatZPanel.add(new JLabel("Z:"));
		JComboBox<String> heatX = new JComboBox<String>();
		JComboBox<String> heatY = new JComboBox<String>();
		JComboBox<String> heatZ = new JComboBox<String>();
		heatX.addActionListener(xA);
		heatY.addActionListener(yA);
		heatZ.addActionListener(zA);
		heatX.setToolTipText("X Axis Series");
		heatY.setToolTipText("Y Axis Series");
		heatZ.setToolTipText("Z Axis Series");
		analysisPanel.add(heatXPanel);
		analysisPanel.add(heatYPanel);
		analysisPanel.add(heatZPanel);
		return analysisPanel;
	}

	/**
	 * @return the confidenceRadio
	 */
	public JRadioButton getConfidenceRadio() {
		return confidenceRadio;
	}

	/**
	 * @param confidenceRadio
	 *            the confidenceRadio to set
	 */
	public void setConfidenceRadio(JRadioButton confidenceRadio) {
		this.confidenceRadio = confidenceRadio;
	}

	/**
	 * @return the addedComponents
	 */
	public LinkedList<JComponent> getAddedComponents() {
		return addedComponents;
	}

	/**
	 * @param addedComponents
	 *            the addedComponents to set
	 */
	public void setAddedComponents(LinkedList<JComponent> addedComponents) {
		this.addedComponents = addedComponents;
	}

	/**
	 * @return the frame
	 */
	public JPanel getFrame() {
		return frame;
	}

	/**
	 * @param frame
	 *            the frame to set
	 */
	public void setFrame(JPanel frame) {
		this.frame = frame;
	}

	/**
	 * @return the visualizationPanel
	 */
	public JPanel getVisualizationPanel() {
		return visualizationPanel;
	}

	/**
	 * @param visualizationPanel
	 *            the visualizationPanel to set
	 */
	public void setVisualizationPanel(JPanel visualizationPanel) {
		this.visualizationPanel = visualizationPanel;
	}

	/**
	 * @return the buttonPanel
	 */
	public JPanel getButtonPanel() {
		return buttonPanel;
	}

	/**
	 * @param buttonPanel
	 *            the buttonPanel to set
	 */
	public void setButtonPanel(JPanel buttonPanel) {
		this.buttonPanel = buttonPanel;
	}

	/**
	 * @return the changeButton
	 */
	public JButton getChangeButton() {
		return changeButton;
	}

	/**
	 * @param changeButton
	 *            the changeButton to set
	 */
	public void setChangeButton(JButton changeButton) {
		this.changeButton = changeButton;
	}

	/**
	 * @return the exportButton
	 */
	public JButton getExportButton() {
		return exportButton;
	}

	/**
	 * @param exportButton
	 *            the exportButton to set
	 */
	public void setExportButton(JButton exportButton) {
		this.exportButton = exportButton;
	}

	/**
	 * @return the chartPanel
	 */
	public ChartPanel getChartPanel() {
		return chartPanel;
	}

	/**
	 * @param chartPanel
	 *            the chartPanel to set
	 */
	public void setChartPanel(ChartPanel chartPanel) {
		this.chartPanel = chartPanel;
	}

	/**
	 * @return the diagramSwitchTypes
	 */
	public JComboBox<String> getDiagramSwitchTypes() {
		return DiagramSwitchTypes;
	}

	/**
	 * @param diagramSwitchTypes
	 *            the diagramSwitchTypes to set
	 */
	public void setDiagramSwitchTypes(JComboBox<String> diagramSwitchTypes) {
		DiagramSwitchTypes = diagramSwitchTypes;
	}



}
